#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

"""Python API for shanbay.com"""

__title__ = 'shanbay'
__version__ = '0.3.3'
__author__ = 'mozillazg'
__email__ = 'mozillazg101@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2014 mozillazg'

import datetime

import requests

from .exceptions import ShanbayException, AuthException, ConnectException
from .message import Message
from .team import Team
from .api import API

all = ['ShanbayException', 'AuthException', 'ConnectException',
       'Shanbay', 'Message', 'Team', 'API']


class Shanbay(object):
    """
    :param username: 用户名
    :param password: 密码

    ::

     >>> from shanbay import Shanbay
     >>> s = Shanbay('username', 'password')
     >>> s.login()
     True
    """
    USER_AGENT = 'python-shanbay/%s' % __version__

    def __init__(self, username, password):
        self._request = requests.Session()
        self.username = username
        self.password = password

    def _attr(self, name):
        return getattr(self.__class__, name)

    def request(self, url, method='get', **kwargs):
        headers = kwargs.setdefault('headers', {})
        headers.setdefault('User-Agent', self._attr('USER_AGENT'))
        try:
            r = getattr(self._request, method)(url, **kwargs)
        except requests.exceptions.RequestException as e:
            raise ConnectException(e)
        if r.url.startswith('http://www.shanbay.com/accounts/login/'):
            raise AuthException('Need login')
        return r

    def login(self, **kwargs):
        """登录"""
        url = 'http://www.shanbay.com/accounts/login/'
        try:
            r = self._request.get(url, **kwargs)
        except requests.exceptions.RequestException as e:
            raise ConnectException(e)
        token = r.cookies.get('csrftoken')
        data = {
            'csrfmiddlewaretoken': token,
            'username': self.username,
            'password': self.password,
            'login': '',
            'continue': 'home',
            'u': 1,
            'next': '',
        }
        return self.request(url, 'post', data=data, **kwargs).ok

    def server_date_utc(self):
        """获取扇贝网服务器时间（UTC 时间）"""
        date_str = self.request('http://www.shanbay.com', 'head'
                                ).headers['date']
        date_utc = datetime.datetime.strptime(date_str,
                                              '%a, %d %b %Y %H:%M:%S GMT')
        return date_utc

    def server_date(self):
        """获取扇贝网服务器时间（北京时间）"""
        date_utc = self.server_date_utc()
        # 北京时间 = UTC + 8 hours
        return date_utc + datetime.timedelta(hours=8)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
import re


class Message(object):
    """站内消息

    :param shanbay: :class:`~shanbay.Shanbay` 实例对象

    ::

     >>> from shanbay import Shanbay, Message
     >>> s = Shanbay('username', 'password')
     >>> s.login()
     >>> m = Message(s)
    """

    def __init__(self, shanbay):
        """ ::

            from shanbay import Shanbay, Message
            s = Shanbay('username', 'password')
            s.login()
            m = Message(s)
        """
        self.shanbay = shanbay
        self._request = shanbay._request
        self.request = shanbay.request

    def send_message(self, recipient_list, subject, body):
        """发送站内消息

        :param recipient_list: 收件人列表
        :param subject: 标题
        :param body: 内容（不能超过 1024 个字符）
        """
        url = 'http://www.shanbay.com/message/compose/'
        recipient = ','.join(recipient_list)
        data = {
            'recipient': recipient,
            'subject': subject,
            'body': body,
            'csrfmiddlewaretoken': self._request.cookies.get('csrftoken')
        }
        response = self.request(url, 'post', data=data)
        return response.url == 'http://www.shanbay.com/message/'

    def reply_message(self, message_url, body):
        """回复某条站内消息

        :param message_url: 该条消息的页面 URL
        :param body: 内容（不能超过 1024 个字符）
        """
        id = re.findall(r'(\d+)/?$', message_url)[0]
        api = 'http://www.shanbay.com/api/v1/message/%s/reply/'
        url = api % id
        data = {
            'body': body
        }
        response = self.request(url, 'post', data=data)
        return response.json()['status_code'] == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
from functools import wraps
import logging

from oauthlib.oauth2 import FatalClientError, OAuth2Error, TokenExpiredError
from requests_oauthlib import OAuth2Session, TokenUpdated

from .exceptions import AuthException

logger = logging.getLogger(__name__)


def _catch_token_error(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except (TokenExpiredError, TokenUpdated, OAuth2Error,
                FatalClientError) as e:
            raise AuthException(e)
    return wrapper


class API(object):
    def __init__(self, client_id, token):
        self.api = OAuth2Session(client_id, token=token)

    def _request(self, url, method='get', params=None, data=None):
        logger.debug('method: {0}'.format(method))
        logger.debug('params: {0}'.format(params))
        logger.debug('data: {0}'.format(data))
        kwargs = {}
        if method in ('get', 'delete'):
            kwargs['params'] = params
        elif method in ('post',):
            kwargs['data'] = data
        logger.debug('kwargs: {0}'.format(kwargs))

        return getattr(self.api, method)(url, **kwargs)

    @_catch_token_error
    def user(self, url='https://api.shanbay.com/account/'):
        """获取用户信息"""
        return self._request(url).json()

    @_catch_token_error
    def word(self, word, url='https://api.shanbay.com/bdc/search/'):
        """查询单词"""
        params = {
            'word': word
        }
        return self._request(url, params=params).json()

    @_catch_token_error
    def add_word(self, word_id, url='https://api.shanbay.com/bdc/learning/'):
        """添加单词"""
        data = {
            'id': word_id
        }
        return self._request(url, method='post', data=data).json()

    @_catch_token_error
    def examples(self, word_id, type=None,
                 url='https://api.shanbay.com/bdc/example/'):
        """获取单词的例句"""
        params = {
            'vocabulary_id': word_id
        }
        if type is not None:
            params['type'] = type
        return self._request(url, params=params).json()

    @_catch_token_error
    def add_example(self, word_id, original, translation,
                    url='https://api.shanbay.com/bdc/example/'):
        """创建例句"""
        data = {
            'vocabulary': word_id,
            'original': original,
            'translation': translation
        }
        return self._request(url, method='post', data=data).json()

    @_catch_token_error
    def favorite_example(self, example_id,
                         url='https://api.shanbay.com/bdc/learning_example/'):
        """收藏例句"""
        data = {
            'example_id': example_id
        }
        return self._request(url, method='post', data=data).json()

    @_catch_token_error
    def delete_example(self, example_id,
                       url='https://api.shanbay.com/bdc/example/{example_id}/'):
        """删除例句"""
        url = url.format(example_id=example_id)
        return self._request(url, method='delete').json()

    @_catch_token_error
    def notes(self, word_id, url='https://api.shanbay.com/bdc/note/'):
        """获取笔记"""
        params = {
            'vocabulary_id': word_id
        }
        return self._request(url, params=params).json()

    @_catch_token_error
    def add_note(self, word_id, note,
                 url='https://api.shanbay.com/bdc/note/'):
        """创建笔记"""
        data = {
            'vocabulary': word_id,
            'note': note
        }
        return self._request(url, method='post', data=data).json()

    @_catch_token_error
    def favorite_note(self, note_id,
                      url='https://api.shanbay.com/bdc/learning_note/'):
        """收藏笔记"""
        data = {
            'note_id': note_id
        }
        return self._request(url, method='post', data=data).json()

    @_catch_token_error
    def delete_note(self, note_id,
                    url='https://api.shanbay.com/bdc/note/{note_id}/'):
        """删除笔记"""
        url = url.format(note_id=note_id)
        return self._request(url, method='delete').json()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals


class ShanbayException(Exception):
    """异常基类"""
    pass


class AuthException(ShanbayException):
    """未登录或登录已过期"""
    pass


class ConnectException(ShanbayException):
    """网络连接出现异常情况"""
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from codecs import open
import re
import sys
import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

requirements = [
    'requests>=2.0.1',
    'beautifulsoup4',
    'html5lib',
    'requests-oauthlib',
]
packages = [
    'shanbay',
]

current_dir = os.path.dirname(os.path.realpath(__file__))


def read_f(name):
    with open(os.path.join(current_dir, name), encoding='utf8') as f:
        return f.read()


def long_description():
    return read_f('README.rst') + '\n\n' + read_f('CHANGELOG.rst')


def meta_info(meta, filename='shanbay/__init__.py', default=''):
    meta = re.escape(meta)
    m = re.search(r"""%s\s+=\s+(?P<quote>['"])(?P<meta>.+?)(?P=quote)""" % meta,
                  read_f(filename))
    return m.group('meta') if m else default

setup(
    name='shanbay',
    version=meta_info('__version__'),
    description='Python wrapper for shanbay.com',
    long_description=long_description(),
    url='https://github.com/mozillazg/python-shanbay',
    download_url='',
    author=meta_info('__author__'),
    author_email=meta_info('__email__'),
    license=meta_info('__license__'),
    packages=packages,
    package_data={'': ['LICENSE']},
    package_dir={'shanbay': 'shanbay'},
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Utilities',
    ],
    keywords='shanbay, 扇贝网',
)
